package org.gridminer.jspace.lifecycle;

import java.io.File;
import java.util.ArrayList;

/**
 * @author Ibrahim Elsayed
 * @version 1.0
 * The LifeCycleResource class represents a single life cycle (experiment)
 * It contains a list with objects from the fife e-science life cycle activities 
 * plus a MetaData object. The fife activities are GoalSpecification, 
 * DataPreparation, TaskSelection, TaskExecution, and ResultProcessing
 *
 */
public class LifeCycleResource {
	
	String name;
	int id;
	ArrayList<LifeCycleActivity> lifeCycleActivities;
	jSpaceConfig JSpaceConfiguration;
	
	//construction without jSpaceConfig object, for testing purposes see the main method below.
	public LifeCycleResource(String name, int id) {
		this.name = name;
		this.id = id;
		lifeCycleActivities = new ArrayList<LifeCycleActivity>();
	}
	
	public LifeCycleResource(String name, int id, jSpaceConfig conf) {
		this.name = name;
		this.id = id;
		this.JSpaceConfiguration = conf;
		lifeCycleActivities = new ArrayList<LifeCycleActivity>();
	}
	
	
	public String getLCRName() {
		return this.name;
	}
	
	public int getLCRID() {
		return this.id;
	}
	
	public jSpaceConfig getConfiguration() {
		return this.JSpaceConfiguration;
	}
	
	public void addMetaInfo(MetaInfo mi) {
		this.lifeCycleActivities.add(0, mi);
	}
	
	public void addGoalSpecificationActivity(GoalSpecification gs) {
		this.lifeCycleActivities.add(1, gs);
	}
	
	public void addDataPreparationActivity(DataPreparation dp) {
		this.lifeCycleActivities.add(2, dp);
	}
	
	public void addTaskSelectionActivity(TaskSelection ts) {
		this.lifeCycleActivities.add(3, ts);
	}
	
	public void addTaskExecutionActivity(TaskExecution te) {
		this.lifeCycleActivities.add(4, te);
	}
	
	public void addResultsProcessingActivity(ResultsProcessing rp) {
		this.lifeCycleActivities.add(5, rp);
	}
	
	//get methods
	public MetaInfo getMetaInfo() {
		MetaInfo mi = (MetaInfo) this.lifeCycleActivities.get(0);
		return mi;
	}
	
	public GoalSpecification getGoalSpecificationActivity() {
		GoalSpecification gs = (GoalSpecification) this.lifeCycleActivities.get(1);
		return gs;
	}
	
	public DataPreparation getDataPreparationActivity() {
		DataPreparation dp = (DataPreparation) this.lifeCycleActivities.get(2);
		return dp;
	}
	
	public TaskSelection getTaskSelectionActivity() {
		TaskSelection ts = (TaskSelection) this.lifeCycleActivities.get(3);
		return ts;
	}
	
	public TaskExecution getTaskExecutionActivity() {
		TaskExecution te = (TaskExecution) this.lifeCycleActivities.get(4);
		return te;
	}
	
	public ResultsProcessing getResultsProcessingActivity() {
		ResultsProcessing rp = (ResultsProcessing) this.lifeCycleActivities.get(5);
		return rp;
	}
	
	
	
	public static void main(String args[]) {
		//create a test LifeCycleResoruce
		LifeCycleResource lcr = new LifeCycleResource("testLCR", 234);
		
		//create the MetaInfo object and attach it to the LifeCycleResource (lcr)
		MetaInfo mi = new MetaInfo(	"k.schwarz", 
									"Konrad Schwarz", 
									"Breath Gas Analysis", 
									"Breath Research Institute of the Austrian Academy of Sciences", 
									"http://www.oeaw.ac.at/aa/", "konrad.schwarz@oeaw.ac.at");
		lcr.addMetaInfo(mi);
		
		//create the e-science life cycle activities
		
		//create a GoalSpecification object (gs)
		String[] gsKeywords = {"trace gas compounds" , "respiratory diseases" , "lung cancer"}; 
		
		GoalSpecification gs = new GoalSpecification(	"A pilot study to define typical characteristics of the trace gas compounds in exhaled breath of non-smokers and smokers to assist interpretation of breath analysis data from patients who smoke with respiratory diseases and lung cancer", 
														gsKeywords, 
														"trace gas compounds");
		
		//attach gs to lcr
		lcr.addGoalSpecificationActivity(gs);
		
		//create a GoalSpecification object (dp)
		String[] dpKeywords = {"PTR-MS" , "smokers" , "non-smokers", "ex-smokers"}; 
		
		DataPreparation dp = new DataPreparation(	"primaryDatasetPath", 
													"fids123.data", 
													"Exhaled breath (analysed using proton PTRMS) for 370 volunteers (81 smokers, 210 non-smokers, 79 ex-smokers)",
													dpKeywords);
		
		//attach dp to lcr
		lcr.addDataPreparationActivity(dp);
		
		//create a TaskSelection object (ts)
		String[] tsKeywords = {"Liliefors test" , "ANOVA" , "Kruskal-Wallis", "receiver-operator-characteristics (ROC) curves"}; 
		
		TaskSelection ts = new TaskSelection(	"backgroundDatasetpath",
												"bdset123.zip",
												"This background dataset includes all MATLAB functions used in the experiment, including Liliefors test, repeated measure analysis of variance (ANOVA, Kruskal-Wallis)",
												tsKeywords);
		
		//attach ts to lcr
		lcr.addTaskSelectionActivity(ts);

		//create a TaskExecution object (te)
		String[] teKeywords = {"mass-to-charge ratios (m/z 28,42,69,79,93,97,123)" , "product ions" , "Youden index (YI = 0.43)", "differentiation smokers and non-smokers"}; 
		
		TaskExecution te = new TaskExecution(	"derivedDatasetPath",
												"ddset123.zip",
												"Volative organic compounds corresponding to product ions at seven mass-to-charge ratios (m/z 28,42,69,79,93,97,123) in the PTR-MS spectra differentiate between smokers and non-smokers.",
												teKeywords);
		
		//attach te to lcr
		lcr.addTaskExecutionActivity(te);
		
		
		//create a Publication Mode
		PublicationMode pubMode = new PublicationMode("community");
		

		//create a ResultsProcessing object (rp)
		ResultsProcessing rp = new ResultsProcessing(	"http://www.iop.org/EJ/article/1752-7163/2/2/026002/jbr8_2_026002.pdf",
														"Validation studies in larger numbers of patients with more precise delineation of their smoking behavior and using additional analytical techniques such as GC-MS and SIFT-MS should be carried out.", pubMode);
		
		//attach rp to lcr
		lcr.addResultsProcessingActivity(rp);
			
		String fileName = "TestLifeCycelResource1.xml";
		File testFile = new File(fileName);
		testFile.delete();
		
		//write the LifeCycleResource Object to an XML file (fileName)
		MyUtilities.saveMyLifeCycleResourceToXMLFile(fileName, lcr);
		
		//create a test LifeCycleResoruce
		LifeCycleResource lcr2 = MyUtilities.getLifeCycleFromXMLFile(fileName);
		
		System.out.println(lcr2.getLCRName() + " " + lcr2.getLCRID());
		
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	

}
