package org.gridminer.jspace.lifecycle.preserver;

import org.gridminer.jspace.lifecycle.LifeCycleResource;
import org.gridminer.jspace.lifecycle.MyUtilities;
import org.gridminer.jspace.manager.PersistentOntology;

import com.hp.hpl.jena.rdf.model.ModelMaker;


/**
 * @author Ibrahim Elsayed
 * @version 1.0
 * The LCRPreserver class preserves a LifeCycleResource Object into a remote database 
 * that is representing a RDF store according to the e-Science life cycle ontology.
 */
public class LCRPreserver {
	
	private LifeCycleResource lcr;
		
	//the ontology URI
    public static final String ONT = "http://www.gridminer.org/e-sciencelifecycle/lifecycleontology_v3.0.owl";

    //local RDF Store Settings
    public String l_dbURL = "";
    public String l_dbUSER = "";
    public String l_dbPASSWD = "";
    public String l_dbType = "";
    public String l_dbDRIVER = "";
	
    //global RDF Store Settings
    public String g_dbURL = "";
    public String g_dbUSER = "";
    public String g_dbPASSWD = "";
    public String g_dbType = "";
    public String g_dbDRIVER = "";
    
    // if true, reload the data
    private static boolean DB_reload = true;
    
    /*
     * Constructor - for instantiation of the LCRPreserver with an XML file containing the Life Cycle Resource and Configuration 
     */
	public LCRPreserver(String fileName) {
		this.lcr = MyUtilities.getLifeCycleFromXMLFile(fileName);
		
		//assign Local RDF Store Seetings to local variables l_dbURL, etc.
		this.l_dbURL = lcr.getConfiguration().getLocalConf()[0];
		this.l_dbUSER = lcr.getConfiguration().getLocalConf()[1];
		this.l_dbPASSWD = lcr.getConfiguration().getLocalConf()[2];
		this.l_dbType = lcr.getConfiguration().getLocalConf()[3];
		this.l_dbDRIVER = lcr.getConfiguration().getLocalConf()[4];
		
		//assign Global RDF Store Seetings to local variables g_dbURL, etc.
		this.g_dbURL = lcr.getConfiguration().getGobalConf()[0];
		this.g_dbUSER = lcr.getConfiguration().getGobalConf()[1];
		this.g_dbPASSWD = lcr.getConfiguration().getGobalConf()[2];
		this.g_dbType = lcr.getConfiguration().getGobalConf()[3];
		this.g_dbDRIVER = lcr.getConfiguration().getGobalConf()[4];
	}
	
	/*
     * Constructor - for instantiation of the LCRPreserver with a given LifeCycleResource Object
     */
	public LCRPreserver(LifeCycleResource lcr) {
		this.lcr = lcr;
		
		//assign Local RDF Store Seetings to local variables l_dbURL, etc.
		this.l_dbURL = lcr.getConfiguration().getLocalConf()[0];
		this.l_dbUSER = lcr.getConfiguration().getLocalConf()[1];
		this.l_dbPASSWD = lcr.getConfiguration().getLocalConf()[2];
		this.l_dbType = lcr.getConfiguration().getLocalConf()[3];
		this.l_dbDRIVER = lcr.getConfiguration().getLocalConf()[4];
		
		//assign Global RDF Store Seetings to local variables g_dbURL, etc.
		this.g_dbURL = lcr.getConfiguration().getGobalConf()[0];
		this.g_dbUSER = lcr.getConfiguration().getGobalConf()[1];
		this.g_dbPASSWD = lcr.getConfiguration().getGobalConf()[2];
		this.g_dbType = lcr.getConfiguration().getGobalConf()[3];
		this.g_dbDRIVER = lcr.getConfiguration().getGobalConf()[4];
		
	}
	
	/*
     * Returns the LifeCycleResource Object 
     */
	public LifeCycleResource getLifeCycleResource() {
		return this.lcr;
	}
	
	/*
     * Returns the LifeCycleResource Object 
     */
	public String getModelID() {
		return this.ONT;
	}
	
	public  ModelMaker getModelMakerFromLocalRDFStore() {
		
		// create the helper class we use to handle the persistent ontologies
        PersistentOntology po = new PersistentOntology();
        
        // ensure the JDBC driver class is loaded
        try {
            Class.forName( l_dbDRIVER );
        }
        catch (Exception e) {
            System.err.println( "Failed to load the driver for the database: " + e.getMessage() );
            System.err.println( "Have you got the CLASSPATH set correctly?" );
        }
        
        // are we re-loading the data this time?
        if (DB_reload) {

            // we pass cleanDB=true to clear out existing models
            // NOTE: this will remove ALL Jena models from the named persistent store, so
            // use with care if you have existing data stored
            ModelMaker maker = po.getRDBMaker( l_dbURL, l_dbUSER, l_dbPASSWD, l_dbType, true );

            // now load the source data into the newly cleaned db
            po.loadDB( maker, ONT );
        }
        
        // now we list the classes in the database, to show that the persistence worked
        ModelMaker maker = po.getRDBMaker( l_dbURL, l_dbUSER, l_dbPASSWD, l_dbType, false );
        
        return maker;
        
	}
	
	public ModelMaker getModelMakerFromGlobalRDFStore() {
		
		// create the helper class we use to handle the persistent ontologies
        PersistentOntology po = new PersistentOntology();
        
        // ensure the JDBC driver class is loaded
        try {
            Class.forName( l_dbDRIVER );
        }
        catch (Exception e) {
            System.err.println( "Failed to load the driver for the database: " + e.getMessage() );
            System.err.println( "Have you got the CLASSPATH set correctly?" );
        }
        
        // are we re-loading the data this time?
        if (DB_reload) {

            // we pass cleanDB=true to clear out existing models
            // NOTE: this will remove ALL Jena models from the named persistent store, so
            // use with care if you have existing data stored
            ModelMaker maker = po.getRDBMaker( l_dbURL, l_dbUSER, l_dbPASSWD, l_dbType, true );

            // now load the source data into the newly cleaned db
            po.loadDB( maker, ONT );
        }
        
        // now we list the classes in the database, to show that the persistence worked
        ModelMaker maker = po.getRDBMaker( l_dbURL, l_dbUSER, l_dbPASSWD, l_dbType, false );
        
        return maker;
        
	}
	
	
	public void publishLCRLocal(PersistentOntology po_l) {
		//publish lcr in the local RDF Store
		po_l.publishLCR(getModelMakerFromLocalRDFStore(), ONT, this.lcr);
	}
	
	
	public void publishLCRGlobal(PersistentOntology po_g) {
		//publish lcr in the global RDF Store
		po_g.publishLCR(getModelMakerFromLocalRDFStore(), ONT, this.lcr);
	}
	
	
}
