package org.gridminer.jspace.manager;


import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Calendar;
import java.util.Iterator;

import org.gridminer.jspace.lifecycle.LifeCycleResource;

import com.hp.hpl.jena.db.*;
import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.sparql.util.IndentedWriter;
import com.hp.hpl.jena.vocabulary.DC;


/**
 * @author Ian Dickinson, HP Labs
 *         (<a  href="mailto:Ian.Dickinson@hp.com" >email</a>)
 * @version CVS $Id: PersistentOntology.java,v 1.1 2005/10/06 17:49:07 andy_seaborne Exp $
 * Extended to support Life Cycle Resources by Ibrahim Elsayed
 */
public class PersistentOntology {

	static public final String NL = System.getProperty("line.separator") ;
  
	public static final String NS = "http://localhost/LifeCycleOntology.owl#";
	public static final String EX3 = "NIGM_CADGrid_001";
	
    public void loadDB( ModelMaker maker, String source ) {
        // use the model maker to get the base model as a persistent model
        // strict=false, so we get an existing model by that name if it exists
        // or create a new one
        Model base = maker.createModel( source, false );

        // now we plug that base model into an ontology model that also uses
        // the given model maker to create storage for imported models
        OntModel m = ModelFactory.createOntologyModel( getModelSpec( maker ), base );

        // now load the source document, which will also load any imports
        System.out.println("Source is: " + source);
        System.out.println("Loading data into local RDF Store ...");
        m.read( source );
    }

    public OntModel getOntModelfromDB(ModelMaker maker, String modelID) {
    	// use the model maker to get the base model as a persistent model
        // strict=false, so we get an existing model by that name if it exists
        // or create a new one
        Model base = maker.createModel( modelID, false );

        // create an ontology model using the persistent model as base
        OntModel m = ModelFactory.createOntologyModel( getModelSpec( maker ), base );
    	
    	return m;
    }
    
  
    public void createN3Format(ModelMaker maker, String modelID) {
    	   Model base = maker.createModel( modelID, false );

           OntModel m = ModelFactory.createOntologyModel( getModelSpec( maker ), base );
           
           m.write( System.out, "N3" );
           
           // add a new timestamp
           Literal now = m.createTypedLiteral( Calendar.getInstance() );
           
           m.getResource( NS + EX3 ).addProperty( DC.date, now );

           Resource r = m.getResource(NS + EX3);
           
           m.commit();
           
           m.close();
           
           Model base2 = maker.createModel( modelID, false );

           OntModel m2 = ModelFactory.createOntologyModel( getModelSpec( maker ), base2 );
           
           m2.write( System.out, "N3" );
                     
           m2.listStatements();
    }
    
    public String serialzeQuery(String q) {
    	Query query = QueryFactory.create(q);
    	
    	return query.serialize();
    }
    
    
    public void executeSparqlQuery(ModelMaker maker, String modelID) {
    	// use the model maker to get the base model as a persistent model
        // strict=false, so we get an existing model by that name if it exists
        // or create a new one
        Model base = maker.createModel( modelID, false );

        //System.out.println("modelID: " + modelID);
        
        // create an ontology model using the persistent model as base
        OntModel m = ModelFactory.createOntologyModel( getModelSpec( maker ), base );
        
        // First part or the query string 
        String prolog = "PREFIX owl: <http://localhost/LifeCycleOntology.owl#>" ;
        
        // Query string.
        String queryString = prolog + NL +
            "SELECT ?lifeCycle ?taskSelection ?scientist ?workflowName WHERE { ?lifeCycle owl:hasTaskSelection ?taskSelection. ?lifeCycle owl:isExecutedBy ?scientist. ?taskSelection owl:hasWorkflow ?workflow. ?workflow owl:workflowName ?workflowName. }" ; 
        
        Query query = QueryFactory.create(queryString) ;
        // Print with line numbers
        query.serialize(new IndentedWriter(System.out,true)) ;
        System.out.println() ;
        
               
        // Create a single execution of this query, apply to a model
        // which is wrapped up as a Dataset
        
        QueryExecution qexec = QueryExecutionFactory.create(query, m) ;
        // Or QueryExecutionFactory.create(queryString, model) ;
        
        System.out.println("LifeCycles: ") ;
        
        try {
            // Assumption: it's a SELECT query.
            ResultSet rs = qexec.execSelect() ;
            
            //System.out.println(rs.toString());
            
            // The order of results is undefined. 
            for ( ; rs.hasNext() ; )
            {
                QuerySolution rb = rs.nextSolution() ;
                                    
                // Get title - variable names do not include the '?' (or '$')
                RDFNode x = rb.get("lifeCycle") ;
                
                //RDFNode x = rb.get("taskSelection") ;
                
                // Check the type of the result value
                if ( x.isLiteral() )
                {
                	Literal titleStr = (Literal) x  ;
                    System.out.println("    " + titleStr) ;
                }
                else
                    System.out.println("not a literal: " + x) ;
            }
        }
        finally
        {
            // QueryExecution objects should be closed to free any system resources 
            qexec.close() ;
        }
        
    }
    
    
    public ResultSet executeTestSparqlQuery(ModelMaker maker, String modelID, String queryString) {
    	
    	ResultSet rs;
    	
    	// use the model maker to get the base model as a persistent model
        // strict=false, so we get an existing model by that name if it exists
        // or create a new one
        Model base = maker.createModel( modelID, false );

        //System.out.println("modelID: " + modelID);
        
        // create an ontology model using the persistent model as base
        OntModel m = ModelFactory.createOntologyModel( getModelSpec( maker ), base );
        
        
        Query query = QueryFactory.create(queryString) ;
        // Print with line numbers
        query.serialize(new IndentedWriter(System.out,true)) ;
        System.out.println() ;
        
               
        // Create a single execution of this query, apply to a model
        // which is wrapped up as a Dataset
        
        QueryExecution qexec = QueryExecutionFactory.create(query, m) ;
        // Or QueryExecutionFactory.create(queryString, model) ;
        
        System.out.println("LifeCycles: ") ;
        
        try {
            // Assumption: it's a SELECT query.
            rs = qexec.execSelect() ;
           
        }
        finally
        {
            // QueryExecution objects should be closed to free any system resources 
            qexec.close() ;
        }
        return rs;
    }
   
    
    /*
     * Executes a SPARQL Construct query 
     */
    public void executeSparqlConstructQuery(ModelMaker maker, String modelID, String queryString) {
    	
    	Model base = maker.createModel(modelID, false);
        OntModel m = ModelFactory.createOntologyModel(getModelSpec(maker), base);
               
        Query query = QueryFactory.create(queryString) ;
        
        query.serialize(new IndentedWriter(System.out,true)) ;
        System.out.println();
        
        QueryExecution qexec = QueryExecutionFactory.create(query, m);
        
        System.out.println("LifeCycles: ");
        
        try {
            // Assumption: it's a SELECT query.
            ResultSet rs = qexec.execSelect();
           
            ResultSetFormatter.out(rs); 
            
            //Model model = rs.getResourceModel();
      
            //model.write(System.out, "RDF/XML-ABBREV");
            
            /*
            // The order of results is undefined. 
            for (; rs.hasNext() ;)
            {
            	QuerySolution rb = rs.nextSolution();
                                    
                // Get title - variable names do not include the '?' (or '$')
                RDFNode x = rb.get("lifeCycle");
                RDFNode y = rb.get("taskSelection");
                
                // Check the type of the result value
                if ( x.isLiteral() )
                {
                	Literal titleStr = (Literal) x;
                    System.out.println("    " + titleStr);
                }
                else
                    System.out.println("not a literal: " + x);
            }
            */
            
        }
        finally
        {
            qexec.close();
        }
       
    }

    /*
     * This method formats the ResultSet according to the formatType. 
     * formatType can either be Text, XML, JSON, or RDF
     * In case it's JSON or RDF the resultSet is written to a file and an empty string is returned.
     * This formats should be chosen for large resultSets. 
     * XML and Text formated resultSets are returned as Strings. 
     */
    public String formatResultSet(ResultSet rs, String formatType, String storeType) {
    	String resultString = "";
    	
    	if (formatType.equalsIgnoreCase("XML")) {
    		resultString = ResultSetFormatter.asXMLString(rs); 
    	}	
    	if (formatType.equalsIgnoreCase("Text")) {
    		resultString = ResultSetFormatter.asText(rs);
    	}
    	if (formatType.equalsIgnoreCase("JSON")) {
    		try {
    			OutputStream output = new FileOutputStream(storeType + "_resultSet_JSON.txt");
        		ResultSetFormatter.outputAsJSON(output, rs);
        		resultString = "JSON output written to file";
    		}
    		catch(IOException e) {
    			e.printStackTrace();
    		}
    	}
    	if (formatType.equalsIgnoreCase("RDF")) {
    		try {
    			OutputStream output = new FileOutputStream(storeType + "_resultSet_RDF.rdf");
        		ResultSetFormatter.outputAsRDF(output, "RDF/XML-ABBREV", rs);
    		}
    		catch(IOException e) {
    			e.printStackTrace();
    		}
    	}
    	//TODO: add other format such as N3
    	
    	return resultString;
    }
    
    
    /*
     * Executes a SPARQL query and returns the ResutlSet object 
     */
    public ResultSet executeSparqlQuery(ModelMaker maker, String modelID, String queryString) {
    	
    	ResultSet rs;
    	
    	Model base = maker.createModel(modelID, false);
        OntModel m = ModelFactory.createOntologyModel(getModelSpec(maker), base);
               
        Query query = QueryFactory.create(queryString) ;
        query.serialize(new IndentedWriter(System.out,true)) ;
        System.out.println();
        
        QueryExecution qexec = QueryExecutionFactory.create(query, m);
        
        try {
            rs = qexec.execSelect();
            
        }
        finally {
            qexec.close();
        }
       
        return rs;
    }

    
    public void listClasses(ModelMaker maker, String modelID) {
        // use the model maker to get the base model as a persistent model
        // strict=false, so we get an existing model by that name if it exists
        // or create a new one
        Model base = maker.createModel(modelID, false);
        
        // create an ontology model using the persistent model as base
        OntModel m = ModelFactory.createOntologyModel( getModelSpec( maker ), base );
         
//        list all individuals in OntModel m    
//        for (Iterator i = m.listIndividuals(); i.hasNext(); ) {
//        	Individual ind = (Individual) i.next();
//            //System.out.println("inListIndividualsIteraator");
//            System.out.println( "Individual " + ind.toString() );
//        }
//        
//        list all classes in OntModel m
//        for (Iterator i = m.listClasses(); i.hasNext(); ) {
//            OntClass c = (OntClass) i.next();
//            //System.out.println("inListClassesIteraator");
//            System.out.println( "Class " + c.getURI() );
//        }
    }
    

    public ModelMaker getRDBMaker(String dbURL, String dbUser, String dbPw, String dbType, boolean cleanDB) {
        try {
            // Create database connection
            IDBConnection conn  = new DBConnection( dbURL, dbUser, dbPw, dbType );

            // do we need to clean the database?
            if (cleanDB) {
                conn.cleanDB();
            }

            // Create a model maker object
            return ModelFactory.createModelRDBMaker( conn );
        }
        catch (Exception e) {
            e.printStackTrace();
            System.exit( 1 );
        }

        return null;
    }

    public OntModelSpec getModelSpec( ModelMaker maker ) {
        // create a spec for the new ont model that will use no inference over models
        // made by the given maker (which is where we get the persistent models from)
        OntModelSpec spec = new OntModelSpec( OntModelSpec.OWL_MEM );
        spec.setImportModelMaker( maker );

        return spec;
    }
    
    /**
     * <p>
     * The Life Cycle Resource will be published into the RDF Store represented by the ModelMaker.
     * New individuals will be created according to the ontology classes.  
     * </p>
     *
     * @param maker ModelMaker representing the 
     * @param modelID String representing the Ontology
     * @param the LifeCycleResource object
     * @return true if successfully published into the RDF Store //TODO 
     */
    public void publishLCR(ModelMaker maker, String modelID, LifeCycleResource lcr) {
    	 
    	System.out.println(modelID);
    	
    	 Model base = maker.createModel( modelID, false );
         OntModel m = ModelFactory.createOntologyModel( getModelSpec( maker ), base );
         
         //create the Resource representing the ontology class to which the individual belongs
         //TODO finish creation of the individuals and properties, use LCRPreserverTester to test if successfully  published into the RDF Store
         //get the values for the individuals from the lcr object. 
        
         String lcrkey = lcr.getLCRName()+lcr.getLCRID();
         
         OntClass lifecycle = m.getOntClass(NS + "lifeCycle");
         Individual lifecycle1 = m.createIndividual(NS + lcrkey, lifecycle);
         
         /***********Meta Info*******************/
         OntClass scientist = m.getOntClass(NS + "scientist"); 
         Individual scientist1 = m.createIndividual(NS + lcr.getMetaInfo().username, scientist);
         
         OntClass researchDomain = m.getOntClass(NS + "researchDomain"); 
         Individual researchDomain1 = m.createIndividual(NS + lcr.getMetaInfo().researchDomain, researchDomain);
                  
         OntClass researchGroup = m.getOntClass(NS + "researchGroup"); 
         Individual researchGroup1 = m.createIndividual(NS + lcr.getMetaInfo().researchGroup, researchGroup);
                 
         /***********Goal Specification*******************/
         OntClass goalSpecification = m.getOntClass(NS + "goalSpecification"); 
         Individual goalSpecification1 = m.createIndividual(NS + lcr.getGoalSpecificationActivity().goalSpecification, goalSpecification);
         
         /***********Data Preparation*******************/
         OntClass dataPreparation = m.getOntClass(NS + "dataPreparation");
         Individual dataPreparation1 = m.createIndividual(NS + lcr.getDataPreparationActivity().dpDescription, dataPreparation);
         
         /***********Task Selection*******************/
         OntClass taskSelection = m.getOntClass(NS + "taskSelection");
         Individual taskSelection1 = m.createIndividual(NS + lcr.getTaskSelectionActivity().tsDescription, taskSelection);
         
         /***********Task Execution*******************/
         OntClass taskExecution = m.getOntClass(NS + "taskExecution");
         Individual taskExecution1 = m.createIndividual(NS + lcr.getTaskExecutionActivity().teDescription, taskExecution);
         
         /***********Result Processing*******************/
         OntClass resultPublishing = m.getOntClass(NS + "taskExecution");
         Individual resultpublishing1 = m.createIndividual(NS + "resultPublishing1", resultPublishing);
         
         OntClass publicationMode = m.getOntClass(NS + "publicationMode");
         Individual publicationMode1 = m.createIndividual(NS + lcr.getResultsProcessingActivity().pubMode.getPubMode(), publicationMode);
         
         OntClass collaboration = m.getOntClass(NS + "collaboration");
         Individual collaboration1 = m.createIndividual(NS + "collaboration-TestLCR-BGA", collaboration);
         
         OntClass lifeCycle = m.getOntClass(NS + "lifeCycle");
         Individual lifeCycle1 = m.createIndividual(NS + "lifeCycle-TestLCR-BGA", lifeCycle); 
         
         /***********EPRs*******************/
         OntClass EPR = m.getOntClass(NS + "EPR");
         Individual EPR1 = m.createIndividual(NS + "EPR1-TestLCR-BGA", EPR); 
         Individual EPR2 = m.createIndividual(NS + "EPR2-TestLCR-BGA", EPR);
         Individual EPR3 = m.createIndividual(NS + "EPR3-TestLCR-BGA", EPR);
         Individual EPR4 = m.createIndividual(NS + "EPR4-TestLCR-BGA", EPR);
         
         /***********metadata*******************/         
         OntClass metadata = m.getOntClass(NS + "metadata");
         Individual metadata1 = m.createIndividual(NS + "metadata1-TestLCR-BGA", metadata); 
         Individual metadata2 = m.createIndividual(NS + "metadata2-TestLCR-BGA", metadata);
         Individual metadata3 = m.createIndividual(NS + "metadata3-TestLCR-BGA", metadata);
         Individual metadata4 = m.createIndividual(NS + "metadata4-TestLCR-BGA", metadata);
         Individual metadata5 = m.createIndividual(NS + "metadata5-TestLCR-BGA", metadata);
         Individual metadata6 = m.createIndividual(NS + "metadata6-TestLCR-BGA", metadata);
         Individual metadata7 = m.createIndividual(NS + "metadata7-TestLCR-BGA", metadata);
         Individual metadata8 = m.createIndividual(NS + "metadata8-TestLCR-BGA", metadata);
         Individual metadata9 = m.createIndividual(NS + "metadata9-TestLCR-BGA", metadata);
         
         /***********participants*******************/         
         OntClass backgroundData = m.getOntClass(NS + "backgroundData");
         Individual backgroundData1 = m.createIndividual(NS + "backgroundData1-TestLCR-BGA", backgroundData); 
         
         OntClass derivedData = m.getOntClass(NS + "derivedData");
         Individual derivedData1 = m.createIndividual(NS + "derivedData1-testLCR-BGA", derivedData);
         Individual derivedData2 = m.createIndividual(NS + "derivedData2-testLCR-BGA", derivedData);
         
         OntClass primaryData = m.getOntClass(NS + "primaryData");
         Individual primaryData1 = m.createIndividual(NS + "primaryData1-testLCR-BGA", primaryData);
        
         
         DatatypeProperty firstName = m.getDatatypeProperty( NS + "firstName" );
         DatatypeProperty lastName = m.getDatatypeProperty( NS + "lastName" );
         DatatypeProperty email = m.getDatatypeProperty( NS + "email" );

         System.out.println();
         
         m.getClass();
         
         for (Iterator i = m.listIndividuals(); i.hasNext(); ) {
             System.out.println("Individual "+ i.next() );
         }
          
    }

}


